/* luafaudes_gen_class.hpp - Lua bindings for Generator class */

/* luafaudes - FAUDES Bindings for Lua

   Copyright (C) 2006 Bernd Opitz
   Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
   at the University of Erlangen-Nuernberg, Germany

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */



#ifndef LUAFAUDES_GEN_CLASS_H
#define LUAFAUDES_GEN_CLASS_H

#include "luafaudes_common.hpp"
#include "luafaudes_eset_class.hpp"

// set faudes eventset we support
typedef faudes::cGenerator fGenerator;
#define LUA_GENERATOR "Generator"

/////////////////////////////////////
// bindings for the Generator class
/////////////////////////////////////

/**
 * Return fGenerator* from lua stack
 *
 * @param L
 *   lua_State*
 * @param index
 *   Index on lua stack
 *
 * @return
 *   fGenerator*
 */
fGenerator* lua_togenerator(lua_State *L, int index);

/**
 * Test whether index on the stack containts a Generator* userdatum
 * and if the pointer contains a valid Generator. If the index is
 * valid, a pointer to the Generator is returned. In the other case
 * an error message is emitted and NULL is returned.
 * 
 * @param L
 *   lua_State*
 * @param index
 *   Index on lua stack
 *
 * @return
 *   fGenerator*; NULL if there is no valid fGenerator*
 */
fGenerator* luaL_checkgenerator(lua_State *L, int index);

/**
 * Test whether index on the stack containts a Generator* userdatum
 * 
 * @param L
 *   lua_State*
 * @param index
 *   Index on lua stack
 *
 * @return
 *   fGenerator*; NULL if there is no valid fGenerator*
 */
bool lua_isgenerator(lua_State* L, int index);

/**
 * Create a new userdata of type LUA_GENERATOR. In the c++ context this 
 * creates a new Generator on the heap and assigns the pointer as the lua 
 * userdatum. Returns the Generator*.
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   fGenerator*
 */
fGenerator* lua_pushgenerator(lua_State *L);

/**
 * (lua user function)
 *
 * Creates new Generator with pushGenerator
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int Generator_New(lua_State *L);

/**
 * Print Generator methods to console
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int Generator_Help(lua_State* L);

/**
 * Garbage collection for Generator userdata values. Calls
 * the delete operator on the Generator.
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int Generator__gc(lua_State *L);

/**
 * (lua user function)
 *
 * (c-) String conversion method for Generators
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int Generator__tostring (lua_State *L);



///////////////////////////
// Generator methods
///////////////////////////


int Generator_Copy(lua_State* L);
int Generator_Write(lua_State* L);
int Generator_DWrite(lua_State* L);
int Generator_StatWrite(lua_State* L);
int Generator_DotWrite(lua_State* L);
int Generator_GraphWrite(lua_State* L);
int Generator_DDotWrite(lua_State* L);
int Generator_AlphabetToString(lua_State* L);
int Generator_StatesToString(lua_State* L);
int Generator_TransRelToString(lua_State* L);
int Generator_InitStatesToString(lua_State* L);
int Generator_MarkedStatesToString(lua_State* L);
int Generator_Read(lua_State* L);
int Generator_Name(lua_State* L);
int Generator_Clear(lua_State* L);
int Generator_ClearStates(lua_State* L);
int Generator_ClearTransRel(lua_State* L);
int Generator_ClearInitStates(lua_State* L);
int Generator_ClearMarkedStates(lua_State* L);
int Generator_AlphabetSize(lua_State* L);
int Generator_Size(lua_State* L);
int Generator_TransRelSize(lua_State* L);
int Generator_InitStatesSize(lua_State* L);
int Generator_MarkedStatesSize(lua_State* L);

int Generator_Empty(lua_State* L);
int Generator_AlphabetEmpty(lua_State* L);
int Generator_TransRelEmpty(lua_State* L);
int Generator_InitStatesEmpty(lua_State* L);
int Generator_MarkedStatesEmpty(lua_State* L);
int Generator_StateNamesEnabled(lua_State* L);
int Generator_InsEvent(lua_State* L);
int Generator_InsEvents(lua_State* L);
int Generator_DelEvent(lua_State* L);
int Generator_DelEvents(lua_State* L);
int Generator_InjectAlphabet(lua_State* L);
int Generator_InsState(lua_State* L);
int Generator_InsInitState(lua_State* L);
int Generator_InsMarkedState(lua_State* L);
int Generator_SetInitState(lua_State* L);
int Generator_ClrInitState(lua_State* L);
int Generator_SetMarkedState(lua_State* L);
int Generator_ClrMarkedState(lua_State* L);
int Generator_DelState(lua_State* L);
int Generator_SetTransition(lua_State* L);
int Generator_ClrTransition(lua_State* L);
int Generator_EventName(lua_State* L);
int Generator_EventIndex(lua_State* L);
int Generator_ExistsEvent(lua_State* L);
int Generator_StateName(lua_State* L);
int Generator_StateIndex(lua_State* L);
int Generator_ExistsState(lua_State* L);
int Generator_ExistsInitState(lua_State* L);
int Generator_ExistsMarkedState(lua_State* L);
int Generator_Alphabet(lua_State* L);
int Generator_MinimizeAlphabet(lua_State* L);
int Generator_UsedEvents(lua_State* L);
int Generator_UnusedEvents(lua_State* L);
int Generator_ActiveEventSet(lua_State* L);
int Generator_SetDefaultStateNames(lua_State* L);
int Generator_ClearStateNames(lua_State* L);
int Generator_Version(lua_State* L);
int Generator_EStr(lua_State* L);
int Generator_SStr(lua_State* L);
int Generator_TStr(lua_State* L);


int Generator_InsControllableEvent(lua_State* L);
int Generator_InsUncontrollableEvent(lua_State* L);
int Generator_SetControllable(lua_State* L);
int Generator_ClrControllable(lua_State* L);
int Generator_Controllable(lua_State* L);
int Generator_ControllableEvents(lua_State* L);
int Generator_UncontrollableEvents(lua_State* L);


//////////////////////////////////
// Generator methods registration
//////////////////////////////////

/**
 * Register LUA_GENERATOR
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int RegisterGeneratorClass(lua_State *L);



#endif
